#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include <assert.h>
#include <errno.h>
#include <signal.h>

int games[6][2] = {
	{ 0 , 1},
	{ 0 , 2},
	{ 0 , 3},
	{ 1 , 2},
	{ 1 , 3},
	{ 2 , 3},
};

int get_goals (int t1, int t2)
{
	char * argv[4];
	char buf[2][10];
	int status;
	pid_t pid;
	argv[0] = "goals";
	argv[1] = buf[0];
	argv[2] = buf[1];
	argv[3] = 0;
	sprintf (buf[0], "%d", t1);
	sprintf (buf[1], "%d", t2);
	pid = fork ();
	if (pid < 0) {
		perror ("fork");
		exit (100);
	}
	if (pid == 0) { /* Child */
		execvp ("goals", argv);
		perror ("execvp");
		*(char *)0 = 0;
		exit (100);
	}
	/* Parent */
	pid = wait (&status);
	if (pid < 0) {
		perror ("wait");
		exit (100);
	}
	if (WIFEXITED (status)) {
		return WEXITSTATUS(status);
	} else {
		fprintf (stderr, "Strange exit\n");
		exit (100);
	}
	return -1;
}

void onegroup (int teams[4], int order[4])
{
	int goals[12];
	char * argv[20];
	char buf[13][10];
	int g, team, i;
	for (g=0; g<6; ++g) {
		for (team=0; team<=1; team++) {
			goals[2*g+team] = get_goals(teams[games[g][team]],
						    teams[games[g][1-team]]);
		}
		printf ("   %2d vs. %-2d  %2d : %-2d\n",
			teams[games[g][0]], teams[games[g][1]],
			goals[2*g+0], goals[2*g+1]);
	}
	argv[0] = "tabelle";
	argv[1] = buf[12];
	for (i=0; i<12; ++i) {
		sprintf (buf[i], "%d", goals[i]);
		argv[2+i] = buf[i];
	}
	argv[14] = 0;
	for (i=0; i<4; ++i) {
		pid_t pid;
		int status;
		sprintf (argv[1], "%d", i);
		pid = fork ();
		if (pid < 0) {
			perror ("fork");
			exit (100);
		}
		if (pid == 0) { /* Child */
			execvp ("tabelle", argv);
			perror ("execvp");
			exit (100);
		}
		/* Parent */
		pid = wait (&status);
		if (pid < 0) {
			perror ("wait");
			exit (100);
		}
		if (WIFEXITED (status)) {
			assert (WEXITSTATUS(status) < 4);
			order[i] = teams[WEXITSTATUS(status)];
		} else {
			if (WIFSIGNALED(status) && WTERMSIG(status)==SIGSEGV) {
				fprintf (stderr, "Would need to draw lots\n");
				exit (1);
			} else {
				fprintf (stderr, "Strange exit\n");
				exit (100);
			}
		}
	}
}

int choose (int t1, int t2)
{
	int g1, g2;
	g1 = g2 = 0;
	while (g1 == g2) {
		g1 = get_goals (t1, t2);
		g2 = get_goals (t2, t1);
		printf (" %2d : %-2d ", g1, g2);
	}
	if (g1 > g2)
		return t1;
	assert (g2 > g1);
	return t2;
}


int main (int argc, char **argv)
{
	int groups[8][4];
	int order[8][4];
	int finalists[16];
	int i,j,t;
	int left;
	t = 0;
	for (i=0; i<8; ++i) {
		for (j=0; j<4; ++j) {
			groups[i][j] = t++;
		}
	}
	for (i=0; i<8; ++i) {
		printf ("Group %c:\n", 'A'+i);
		onegroup (groups[i], order[i]);
		printf ("Group %c: Order: %d %d %d %d\n", 'A'+i,
		        order[i][0], order[i][1], order[i][2], order[i][3]);
		if (i%2 ==0) {
			finalists[i] = order[i][0];
			finalists[i+8] = order[i][1];
		} else {
			finalists[i] = order[i][1];
			finalists[i+8] = order[i][0];
		}
	}
	for (left=16; left > 1; left /= 2) {
		int k = 0;
		printf ("Round of last %d teams:\n", left);
		fflush (stdout);
		for (i=0; i<left; i+=2) {
			assert (left == 2 || finalists[i]/4 != finalists[i+1]/4);
			printf ("   %2d vs. %-2d: ",
				finalists[i], finalists[i+1]);
			int winner = choose (finalists[i], finalists[i+1]);
			printf ("Winner: %2d\n", winner);
			finalists[k++] = winner;
		}
	}
	return 0;
}
